# archiepixa


`archiepixa` is a minimalist Bash script designed to visualize archives with a webserver. It scans a specified folder and its subfolders, compresses previews using `ffmpeg` and `imagemagick`, and compiles them into HTML files, one for each subfolder. 

The code has been developed for the workshop *[Building collective digital archives]("https://dsignweek.servus.at/program/fri-24/building-collective-digital-archives/")* during the [D*sign Week 2024, Linz]("https://dsignweek.servus.at/"). Powered by [Servus]("https://servus.at")


## Workflow

- Scans specified folder and subfolders for files
- Compresses previews for images, videos, and audio files
- Generates static HTML files with customizable CSS styling
- Customization of main color, secondary color, background color, and footer text


## System Requirements

- Bash shell
- `ffmpeg` and `imagemagick` packages for media file compression

## Installation Instructions

### Installing ffmpeg and imagemagick

To install `ffmpeg` and `imagemagick`, use the package manager specific to your operating system.

#### For Ubuntu/Debian:

```bash
sudo apt-get install ffmpeg imagemagick
```

#### For macOS (using Homebrew):

```bash
brew install ffmpeg imagemagick
```


## Usage

To use `archiepixa`, follow these steps:

1. Place `archiepixa.sh` in the parent folder of the directory you want to scan.
2. Run the script using the following command:

```bash
bash archiepixa.sh -f <folder_name>
```

Replace `<folder_name>` with the name of the folder you want to scan.

## Getting started

A practical example:

```bash
cd /var/www/html/
git clone http://git.servus.at/servus/archiepixa
mv archiepixa/archipiexa.sh ./
bash archiepixa.sh -f ""
```

## Customization

You can customize the appearance of the generated HTML files by modifying the **CSS** within the Bash script.

You can use flags to modify the background, the main, the second, the third color and you can choose how many colors the dither uses.

```bash
bash archiepixa.sh -f "folder" -b "black" -1 "purple" -2 "rgb('22','45','223')" -3 "#342298" -d "5" -F "The footer of the archiepixa magic"
```

Defaults are:

* -1 "red"
* -2 "orange"
* -3 "black"
* -b "white"
* -d "3"
* -F none


## Cleanup

To remove the generated HTML files, use the `-r` flag along with the folder name:

```bash
bash archiepixa.sh -f <folder_name> -r
```

To delete both the HTML files and compressed folders, use the `-R` flag:

```bash
bash archiepixa.sh -f <folder_name> -R
```

## IMPORTANT!

**Note:** Avoid placing files named "index.html" within the scanned folders to prevent conflicts.

Do **ALWAYS** a backup of what you are scanning with archiepixa.


## License

This script is licensed under the [Habitat License](https://habitattt.it/license).

## Acknowledgments

